cdef extern from "lib/gfxdevice.h":
    ctypedef enum:
        gfx_moveTo
        gfx_lineTo
        gfx_splineTo
        gfx_linetype

    ctypedef double gfxcoord_t

    ctypedef enum gfx_linetype:
        gfx_moveTo
        gfx_lineTo
        gfx_splineTo
    ctypedef enum gfx_joinType:
        gfx_joinMiter
        gfx_joinRound
        gfx_joinBevel
    ctypedef enum gfx_capType:
        gfx_capButt
        gfx_capRound
        gfx_capSquare

    ctypedef struct gfxline_t:
        gfx_linetype type
        gfxcoord_t x,y
        gfxcoord_t sx,sy
        gfxline_t*next # NULL=end

    ctypedef struct gfxglyph_t:
        gfxline_t*line
        gfxcoord_t advance
        int unicode # array?
        char*name

    ctypedef struct gfxkerning_t:
        int c1,c2,advance

    ctypedef struct gfxfont_t:
        char*id
        int num_glyphs
        int max_unicode
        double ascent,descent
        gfxglyph_t*glyphs
        int* unicode2glyph
        gfxkerning_t*kerning
        int kerning_size

    ctypedef struct gfxcolor_t:
        unsigned char a
        unsigned char r
        unsigned char g
        unsigned char b

    ctypedef struct gfxmatrix_t:
        double m00,m10,tx
        double m01,m11,ty

    ctypedef struct gfximage_t:
        # if the data contains an alpha layer (a != 255), the
        #  r,g,b values will have to be premultiplied
        gfxcolor_t*data
        int width
        int height

    ctypedef enum gfxgradienttype_t:
        gfxgradient_radial
        gfxgradient_linear

    ctypedef struct gfxgradient_t:
        gfxcolor_t color
        float pos # 0.0 - 1.0
        gfxgradient_t*next # NULL=end

    ctypedef struct gfxcxform_t:
        float rr,rg,rb,ra, tr
        float gr,gg,gb,ga, tg
        float br,bg,bb,ba, tb
        float ar,ag,ab,aa, ta

    ctypedef struct gfxbbox_t:
        gfxcoord_t xmin, ymin, xmax, ymax

    ctypedef struct gfxresult_t:
        void (*write)(gfxresult_t*gfx, int filedesc)
        int (*save)(gfxresult_t*gfx, char*filename)
        void* (*get)(gfxresult_t*gfx, char*name) nogil
        void (*destroy)(gfxresult_t*gfx) nogil
        void*internal

    ctypedef struct gfxdevice_t:
        char* name # gfx device name
        int (*setparameter)(gfxdevice_t*dev, char*key, char*value) nogil
        void (*startpage)(gfxdevice_t*dev, int width, int height) nogil
        void (*startclip)(gfxdevice_t*dev, gfxline_t*line)
        void (*endclip)(gfxdevice_t*dev)
        void (*stroke)(gfxdevice_t*dev, gfxline_t*line, gfxcoord_t width, gfxcolor_t*color, gfx_capType cap_style, gfx_joinType joint_style, gfxcoord_t miterLimit)
        void (*fill)(gfxdevice_t*dev, gfxline_t*line, gfxcolor_t*color)
        void (*fillbitmap)(gfxdevice_t*dev, gfxline_t*line, gfximage_t*img, gfxmatrix_t*imgcoord2devcoord, gfxcxform_t*cxform) # cxform? tiling?
        void (*fillgradient)(gfxdevice_t*dev, gfxline_t*line, gfxgradient_t*gradient, gfxgradienttype_t type, gfxmatrix_t*gradcoord2devcoord) # ?
        void (*addfont)(gfxdevice_t*dev, gfxfont_t*font) # deprecated 
        void (*drawchar)(gfxdevice_t*dev, gfxfont_t*font, int glyph, gfxcolor_t*color, gfxmatrix_t*matrix)
        void (*drawlink)(gfxdevice_t*dev, gfxline_t*line, char*action)
        void (*endpage)(gfxdevice_t*dev) nogil
        char* (*geterror)()
        gfxresult_t* (*finish)(gfxdevice_t*dev) nogil
        void* internal

cdef extern from "lib/gfxsource.h":

    ctypedef struct gfxdocument_t
    ctypedef struct gfxpage_t

    ctypedef struct gfxsource_t:
        void (*setparameter)(gfxsource_t*src, char*name, char*value)
        gfxdocument_t* (*open)(gfxsource_t*src, char*filename) nogil
        void (*destroy)(gfxsource_t*src)
        void*internal

    ctypedef struct gfxdocument_t:
        gfxsource_t*driver
        int num_pages
        void* (*get)(gfxdocument_t*gfx, char*name)
        void (*destroy)(gfxdocument_t*gfx)
        void (*setparameter)(gfxdocument_t*gfx, char*name, char*value)
        char* (*getinfo)(gfxdocument_t*gfx, char*key)
        gfxpage_t* (*getpage)(gfxdocument_t*gfx, int page)
        void (*prepare)(gfxdocument_t*gfx, gfxdevice_t*device)
        void*internal

    ctypedef struct gfxpage_t:
        gfxdocument_t*parent
        gfxcoord_t width
        gfxcoord_t height
        int nr
        void (*render)(gfxpage_t*gfx, gfxdevice_t*output) nogil
        void (*rendersection)(gfxpage_t*gfx, gfxdevice_t*output, gfxcoord_t x, gfxcoord_t y, gfxcoord_t x1, gfxcoord_t y1, gfxcoord_t x2, gfxcoord_t y2)
        void (*destroy)(gfxpage_t*page)
        void*internal

cdef extern from "lib/pdf/pdf.h":
    gfxsource_t* gfxsource_pdf_create()

cdef extern from "lib/devices/swf.h":
    void gfxdevice_swf_init(gfxdevice_t*)

cdef extern from "lib/devices/ocr.h":
    void gfxdevice_ocr_init(gfxdevice_t*)

cdef extern from "lib/devices/text.h":
    void gfxdevice_text_init(gfxdevice_t*)

cdef extern from "lib/devices/render.h":
    void gfxdevice_render_init(gfxdevice_t*) nogil

cdef extern from "lib/devices/pdf.h":
    void gfxdevice_pdf_init(gfxdevice_t*)

cdef extern from "lib/devices/rescale.h":
    void gfxdevice_rescale_init(gfxdevice_t*dev, gfxdevice_t*out, int width, int height, double scale) nogil

cdef extern from "lib/log.h":
    extern void setConsoleLogging(int level)

cdef extern from *:
    ctypedef char* const_char_ptr "const char*"
